/*
* RTP/RTCP H264/MPEG-4 AAC Encoder
* Copyright (c) 2008 Marius Gligor
*
* Author: Marius Gligor <marius.gligor@gmail.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
*/
//---------------------------------------------------------------------------
#include <vcl.h>
#include <stdio.h>
#pragma hdrstop

#include "Main.h"
#include "Protection.h"

#pragma package(smart_init)
#pragma resource "*.dfm"

TMainForm *MainForm;
TProtectionForm *Secure;
WSADATA wsaData;
//---------------------------------------------------------------------------
static int WW[] = {160, 176, 320, 352, 360, 360, 512, 640, 704, 720, 720, 1280, 1920};
static int HH[] = {120, 144, 240, 288, 240, 288, 288, 480, 576, 480, 576,  720, 1080};
//---------------------------------------------------------------------------
AnsiString __fastcall crypto(AnsiString txt)
{
  char *b = txt.c_str();
  for(int i = 0;i < txt.Length();i++)
  {
    b[i] ^= 0xB5;
  }
  return txt;
}
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
  ::CoInitialize(NULL);
  // network
  ::WSAStartup(MAKEWORD(2, 0), &wsaData);
  Secure = new TProtectionForm(this);

  Application->Title = Caption;
  TrayIcon->Hint = Caption;

  DoubleBuffered = true;
  Renderer->DoubleBuffered = true;

  graf = NULL;
  rtsp = NULL;

  //video
  VideoSources->Items->Clear();
  VideoSources->Items->Add("(none)");
  CDevice::GetVideoSources(VideoSources->Handle);
  VideoSources->ItemIndex = 0;
  //audio
  AudioSources->Items->Clear();
  AudioSources->Items->Add("(none)");
  CDevice::GetAudioSources(AudioSources->Handle);
  AudioSources->ItemIndex = 0;

  Image->Picture = Image2->Picture;
  LoadParams();
  OnSelectSources(NULL);
  OnSettings(NULL);
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FormCreate(TObject *Sender)
{
  if(Autorun->Checked)
  {
    OnStart(NULL);
  }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::OnClose(TObject *Sender, TCloseAction &Action)
{
  CleanUp();
  SaveParams();
  delete Secure;
}
//---------------------------------------------------------------------------

int __fastcall TMainForm::InitializeContext()
{
  // alloc context
  memset(&cntx, 0, sizeof(cntx));
  logo.SetPosition(GetPosition());
  logo.SetOpaque(Opaque->Checked);
  cntx.draw = DrawLogo->Checked;
  cntx.logo = &logo;
  cntx.draw = DrawLogo->Checked;
  // rtp
  cntx.mode = Mode->ItemIndex;
  cntx.ttl = TTL->Position;
  strcpy(cntx.v_host, RtpHost->Text.c_str());
  cntx.v_port = RtpPort->Position;
  strcpy(cntx.a_host, cntx.v_host);
  cntx.a_port = cntx.v_port + 2;
  cntx.cast = (IsMulticast(RtpHost->Text.c_str()) == IP_MULTICAST);
  // rtsp
  MakeSDP();
  strcpy(cntx.host, Host->Text.c_str());
  cntx.port = Port->Position;
  strcpy(cntx.path, Path->Text.c_str());
  strcpy(cntx.sdp, sdp);
  strcpy(cntx.username, usr->Text.c_str());
  strcpy(cntx.password, psw->Text.c_str());
  // video
  cntx.preview = Preview->Checked;
  cntx.v = VideoSources->ItemIndex;
  _tcscpy(cntx.video_source, WideString(VideoSources->Text).c_bstr());
  _tcscpy(cntx.audio_source, WideString(AudioSources->Text).c_bstr());
  cntx.frame_rate = FrameRate->Position;
  cntx.key_frame_rate = KeyFrame->Position;
  cntx.v_bit_rate = VideoDataRate->Position;
  cntx.hWnd = Renderer->Handle;
  cntx.width = W->Position;
  cntx.height = H->Position;
  GetRendererSize();
  cntx.preset = Preset->ItemIndex;
  cntx.tune = Tune->ItemIndex;
  cntx.tune1 = Tune1->ItemIndex;
  cntx.tune2 = Tune2->ItemIndex;
  cntx.profile = Profile->ItemIndex;
  cntx.change_rate = ChangeRate->Checked;
  // audio
  cntx.a = AudioSources->ItemIndex;
  cntx.wfx = GetAudioFormat();
  cntx.a_bit_rate = 1000 * AudioDataRate->Position;
  cntx.a_band_width = 0; // cutoff in Hz cntx.wfx->nSamplesPerSec;
  cntx.a_obj = ObjectType->ItemIndex + 1;
  // session
  if(!rtsp)
  {
    rtsp = new CSess();
    z = rtsp->Open(&cntx);
    StatusBar->Panels->Items[3]->Text = IntToStr(z);
    if(z == HTTP_OK)
    {
      // direct show
      StatusBar->Panels->Items[4]->Text = "OK";
      cntx.mtu = 6;
      cntx.v_call = new CVideoIn(&cntx);
      cntx.a_call = new CAudioIn(&cntx);
      VideoBtn->Enabled = VideoSources->ItemIndex;
      AudioBtn->Enabled = AudioSources->ItemIndex;
    }
    else if(z == HTTP_AUTH)
    {
      StatusBar->Panels->Items[4]->Text = "Unauthorized";
    }
    else if(cntx.ok)
    {
      ErrorMessage(cntx.ok);
    }
  }
  return z;
}
//---------------------------------------------------------------------------

int __fastcall TMainForm::Start(AnsiString txt)
{
  StatusBar->Panels->Items[4]->Text = txt;
  StartBtn->Caption = "&Stop";
  StartBtn->Hint = "stop capture";
  StartBtn->Tag = 1;
  StartUp();
  return z;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::Stop(AnsiString txt)
{
  CleanUp();
  StartBtn->Caption = "&Start";
  StartBtn->Hint = "start capture";
  StartBtn->Tag = 0;
  StatusBar->Panels->Items[4]->Text = txt;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::OnStart(TObject *Sender)
{
  WatchDog->Enabled = false;
  StartBtn->Tag = !StartBtn->Tag;
  if(StartBtn->Tag)
  {
    GmaxLogo->Visible = !Preview->Checked;
    Start("");
    Timer->Enabled = true;
  }
  else
  {
    // stop
    GmaxLogo->Visible = true;
    Timer->Enabled = false;
    Timer->Tag = 0;
    Stop("");
  }
  WatchDog->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::StartUp()
{
  StartBtn->Enabled = false;
  try
  {
    if(!graf && InitializeContext() == HTTP_OK)
    {
      graf = new CCapture();
      if(graf->InitGraph(cntx))
      {
        graf->Start();
      }
      else
      {
        Memo->Lines->Add(graf->GetLastError());
      }
    }
  }
  catch(Exception &e)
  {
    Memo->Lines->Add(e.Message);
  }
  StartBtn->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::CleanUp()
{
  VideoBtn->Enabled = false;
  AudioBtn->Enabled = false;
  StartBtn->Enabled = false;
  try
  {
    // stop
    if(graf)
    {
      graf->Stop();
      delete graf;
      graf = NULL;
    }
    // audio
    if(cntx.a_call)
    {
      delete cntx.a_call;
      cntx.a_call = NULL;
    }
    // video
    if(cntx.v_call)
    {
      delete cntx.v_call;
      cntx.v_call = NULL;
    }
  }
  catch(Exception &e)
  {
    Memo->Lines->Add(e.Message);
  }

  if(rtsp)
  {
    z = rtsp->Close();
    delete rtsp;
    rtsp = NULL;
    StatusBar->Panels->Items[3]->Text = IntToStr(z);
  }
  StartBtn->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::OnVideo(TObject *Sender)
{
  if(graf)
  {
    graf->VideoProperty(Renderer->Handle);
  }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::OnAudio(TObject *Sender)
{
  if(graf)
  {
    graf->AudioProperty(Renderer->Handle);
  }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::LoadParams()
{
  Load();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::SaveParams()
{
  Save();
}
//---------------------------------------------------------------------------

void TMainForm::Save()
{
  TRegistry *reg = new TRegistry();
  try
  {
    reg->Access = KEY_ALL_ACCESS;
    reg->RootKey = HKEY_CURRENT_USER;
    reg->OpenKey("\\SOFTWARE\\Gmax\\RTSP Encoder", true);
    reg->WriteBool("Preview", Preview->Checked);
    reg->WriteString("Server", Host->Text);
    reg->WriteInteger("Port", Port->Position);
    reg->WriteString("Path", Path->Text);
    reg->WriteString("RtpServer", RtpHost->Text);
    reg->WriteInteger("RtpPort", RtpPort->Position);
    reg->WriteInteger("TTL", TTL->Position);
    reg->WriteInteger("Mode", Mode->ItemIndex);
    reg->WriteInteger("VideoSource", VideoSources->ItemIndex);
    reg->WriteInteger("AudioSource", AudioSources->ItemIndex);
    reg->WriteInteger("AudioSampleRate", SampleRate->ItemIndex);
    reg->WriteInteger("AudioBitRate", AudioDataRate->Position);
    reg->WriteInteger("Channels", Channels->ItemIndex);
    reg->WriteInteger("ObjectType", ObjectType->ItemIndex);
    reg->WriteInteger("Width", W->Position);
    reg->WriteInteger("Height", H->Position);
    reg->WriteInteger("VideoFrameRate", FrameRate->Position);
    reg->WriteInteger("VideoBitRate", VideoDataRate->Position);
    reg->WriteInteger("KeyFrameRate", KeyFrame->Position);
    reg->WriteInteger("Preset", Preset->ItemIndex);
    reg->WriteInteger("Tune", Tune->ItemIndex);
    reg->WriteInteger("Tune1", Tune1->ItemIndex);
    reg->WriteInteger("Tune2", Tune2->ItemIndex);
    reg->WriteInteger("Profile", Profile->ItemIndex);
    reg->WriteBool("LogoEnabled", DrawLogo->Checked);
    reg->WriteBool("LogoOpaque", Opaque->Checked);
    reg->WriteInteger("LogoPosition", GetPosition());
    reg->WriteString("LogoFile", OpenPictureDialog->FileName);
    reg->WriteString("Username", usr->Text);
    AnsiString p = psw->Text;
    reg->WriteString("Password", crypto(p));
    reg->WriteBool("Autorun", Autorun->Checked);
    reg->WriteBool("Watchdog", WatchdogEnabled->Checked);
    reg->WriteInteger("WatchdogTime", WatchdogTime->Position);
    reg->WriteBool("ChangeRate", ChangeRate->Checked);
    reg->CloseKey();
  }
  catch(...)
  {
  }
  delete reg;
 }
//---------------------------------------------------------------------------

void TMainForm::Load()
{
  TRegistry *reg = new TRegistry();
  try
  {
    reg->Access = KEY_ALL_ACCESS;
    reg->RootKey = HKEY_CURRENT_USER;
    reg->OpenKey("\\SOFTWARE\\Gmax\\RTSP Encoder", true);
    Preview->Checked = reg->ReadBool("Preview");
    Host->Text = reg->ReadString("Server");
    Port->Position = reg->ReadInteger("Port");
    Path->Text = reg->ReadString("Path");
    RtpHost->Text = reg->ReadString("RtpServer");
    RtpPort->Position = reg->ReadInteger("RtpPort");
    TTL->Position = reg->ReadInteger("TTL");
    Mode->ItemIndex = reg->ReadInteger("Mode");
    VideoSources->ItemIndex = reg->ReadInteger("VideoSource");
    AudioSources->ItemIndex = reg->ReadInteger("AudioSource");
    SampleRate->ItemIndex = reg->ReadInteger("AudioSampleRate");
    AudioDataRate->Position = reg->ReadInteger("AudioBitRate");
    Channels->ItemIndex = reg->ReadInteger("Channels");
    ObjectType->ItemIndex = reg->ReadInteger("ObjectType");
    W->Position = reg->ReadInteger("Width");
    H->Position = reg->ReadInteger("Height");
    FrameRate->Position = reg->ReadInteger("VideoFrameRate");
    VideoDataRate->Position = reg->ReadInteger("VideoBitRate");
    KeyFrame->Position = reg->ReadInteger("KeyFrameRate");
    Preset->ItemIndex = reg->ReadInteger("Preset");
    Tune->ItemIndex = reg->ReadInteger("Tune");
    Tune1->ItemIndex = reg->ReadInteger("Tune1");
    Tune2->ItemIndex = reg->ReadInteger("Tune2");
    Profile->ItemIndex = reg->ReadInteger("Profile");
    DrawLogo->Checked = reg->ReadBool("LogoEnabled");
    Opaque->Checked = reg->ReadBool("LogoOpaque");
    Transparent->Checked = !Opaque->Checked;
    SetPosition(reg->ReadInteger("LogoPosition"));
    OpenPictureDialog->FileName = reg->ReadString("LogoFile");
    if(FileExists(OpenPictureDialog->FileName))
    {
      logo.LoadLogo(WideString(OpenPictureDialog->FileName).c_bstr());
      DrawImage();
    }
    usr->Text = reg->ReadString("Username");
    AnsiString p = reg->ReadString("Password");
    psw->Text = crypto(p);
    Autorun->Checked = reg->ReadBool("Autorun");
    WatchdogTime->Position = reg->ReadInteger("WatchdogTime");
    WatchDog->Interval = WatchdogTime->Position * 1000;
    WatchdogEnabled->Checked = reg->ReadBool("Watchdog");
    WatchDog->Enabled = WatchdogEnabled->Checked;
    ChangeRate->Checked = reg->ReadBool("ChangeRate");
    reg->CloseKey();
  }
  catch(...)
  {
  }
  delete reg;
 }
//---------------------------------------------------------------------------

void TMainForm::ErrorMessage(DWORD dwError)
{
  LPTSTR lpMessage;
  // get message
  StatusBar->Panels->Items[4]->Text = IntToStr(dwError);
  if(::FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_ALLOCATE_BUFFER,
    NULL, dwError, 0, (LPTSTR)&lpMessage, 0, NULL))
  {
    // eliminate CR and LF
    for(int i = _tcslen(lpMessage) - 1;i >= 0;i--)
    {
      if(lpMessage[i] == _T('\n')) lpMessage[i] = 0;
      else if(lpMessage[i] == _T('\r')) lpMessage[i] = 0;
    }
    Memo->Lines->Add(lpMessage);
    ::LocalFree(lpMessage);
  }
  else
  {
    Memo->Lines->Add("Unknown error.");
  }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::OnTimer(TObject *Sender)
{
  Timer->Enabled = false;
  AnsiString tmp;
  // time
  Timer->Tag++;
  int ss = Timer->Tag % 60;
  int mm = (Timer->Tag / 60) % 60;
  int hh = (Timer->Tag / 3600) % 24;
  tmp.printf("%02u:%02u:%02u", hh, mm, ss);
  StatusBar->Panels->Items[0]->Text = tmp;
  // bandwidth
  tmp.printf("%.2f", cntx.audio_bw);
  StatusBar->Panels->Items[2]->Text = tmp;
  tmp.printf("%.2f", cntx.video_bw);
  StatusBar->Panels->Items[1]->Text = tmp;
  Timer->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::OnSdp(TObject *Sender)
{
  if(SaveDialog->Execute())
  {
    MakeSDP();
    FILE *fp = fopen(SaveDialog->FileName.c_str(), "wt");
    if(fp)
    {
      fputs(sdp, fp);
      fclose(fp);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::MakeSDP()
{
  char fp[128];
  strcpy(sdp, "/* SDP */\n");
  strcat(sdp, "v=0\n");
  strcat(sdp, "o=- 20090101 20201231 IN IP4 www.gmax.ws\n");
  strcat(sdp, "s=Gmax RTSP/RTP/RTCP H.264/MPEG4-AAC Encoder\n");
  strcat(sdp, "t=0 0\n");
  if(VideoSources->ItemIndex)
  {
    // video
    strcat(sdp, "/* video */\n");
    sprintf(fp, "m=video %u RTP/AVP %u\n", RtpPort->Position, AVC_PAYLOAD_TYPE);
    strcat(sdp, fp);
    sprintf(fp, "a=rtpmap:%u H264/90000\n", AVC_PAYLOAD_TYPE);
    strcat(sdp, fp);
    sprintf(fp, "a=framesize:%u %u-%u\n", AVC_PAYLOAD_TYPE, W->Position, H->Position);
    strcat(sdp, fp);
    strcat(sdp, "a=control:trackID=1\n");
    sprintf(fp, "b=AS:%u\n", VideoDataRate->Position);
    strcat(sdp, fp);
    if(IsMulticast(RtpHost->Text.c_str()) == IP_MULTICAST)
    {
      sprintf(fp, "c=IN IP4 %s/%u\n", RtpHost->Text.c_str(), TTL->Position);
      strcat(sdp, fp);
    }
  }
  if(AudioSources->ItemIndex)
  {
    strcat(sdp, "/* audio */\n");
    LPWAVEFORMATEX wfx = GetAudioFormat();
    sprintf(fp, "m=audio %u RTP/AVP %u\n", RtpPort->Position + 2, AAC_PAYLOAD_TYPE);
    strcat(sdp, fp);
    sprintf(fp, "a=rtpmap:%u MP4A-LATM/%u/%u\n",
      AAC_PAYLOAD_TYPE, wfx->nSamplesPerSec, wfx->nChannels);
    strcat(sdp, fp);

    char config[16];
    switch(wfx->nSamplesPerSec)
    {
      case 11025:
        strncpy(config, "40001A103FC0", 12);
        break;
      case 22050:
        strncpy(config, "400017103FC0", 12);
        break;
      case 32000:
        strncpy(config, "400025103FC0", 12);
        break;
      case 44100:
        strncpy(config, "400014103FC0", 12);
        break;
      case 48000:
        strncpy(config, "400013103FC0", 12);
        break;
    }
    config[12] = 0;
    if(wfx->nChannels == 2)
    {
      config[6]++;
    }
    config[4] += ObjectType->ItemIndex;

    sprintf(fp, "a=fmtp:%u profile-level-id=1; object=%u; cpresent=0; config=%s\n",
      AAC_PAYLOAD_TYPE, ObjectType->ItemIndex + 1, config);
    strcat(sdp, fp);
    strcat(sdp, "a=control:trackID=2\n");
    sprintf(fp, "b=AS:%u\n", AudioDataRate->Position);
    strcat(sdp, fp);
    if(IsMulticast(RtpHost->Text.c_str()) == IP_MULTICAST)
    {
      sprintf(fp, "c=IN IP4 %s/%u\n", RtpHost->Text.c_str(), TTL->Position);
      strcat(sdp, fp);
    }
  }
}
//---------------------------------------------------------------------------

int TMainForm::IsMulticast(char *dest)
{
  uint32_t ip = inet_addr(dest);
  // invalid address
  if(ip == INADDR_NONE)
  {
    return IP_ERROR;
  }
  // multicast
  if((ip & IP_MULTICAST_MASK) == IP_MULTICAST_BYTE)
  {
    return IP_MULTICAST;
  }
  // broadcast / unicast
  return ((ip & IP_BROADCAST_MASK) == IP_BROADCAST_MASK) ? IP_BROADCAST : IP_UNICAST;
}
//---------------------------------------------------------------------------
// return audio format
LPWAVEFORMATEX TMainForm::GetAudioFormat()
{
  ZeroMemory(&wfx, sizeof(wfx));
  wfx.wFormatTag = WAVE_FORMAT_PCM;
  wfx.nChannels = Channels->ItemIndex + 1;
  wfx.nSamplesPerSec = StrToInt(SampleRate->Text);;
  wfx.wBitsPerSample = 16;
  wfx.nBlockAlign = 2 * wfx.nChannels;
  wfx.nAvgBytesPerSec = wfx.nSamplesPerSec * wfx.nBlockAlign;
  return &wfx;
}
//---------------------------------------------------------------------------

void TMainForm::GetRendererSize()
{
  if(W->Position > 704 || H->Position > 576)
  {
    cntx.r.right = W->Position / 4;
    cntx.r.bottom = H->Position / 4;
  }
  else if(W->Position > 352 || H->Position > 288)
  {
    cntx.r.right = W->Position / 2;
    cntx.r.bottom = H->Position / 2;
  }
  else
  {
    cntx.r.right = W->Position;
    cntx.r.bottom = H->Position;
  }
  cntx.r.left = 1 + (352 - cntx.r.right) / 2;
  cntx.r.top = 1 + (288 - cntx.r.bottom) / 2;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::OnResolution(TObject *Sender)
{
  int i = ((TMenuItem *)Sender)->Tag;
  W->Position = WW[i];
  H->Position = HH[i];
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::OnDestroy(TObject *Sender)
{
  ::WSACleanup();
  ::CoUninitialize();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::OnLock(TObject *Sender)
{
  if(Secure->SetPassword())
  {
    Visible = false;
    TrayIcon->Visible = !Visible;
  }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::OnUnlock(TObject *Sender)
{
  if(Secure->GetPassword())
  {
    Visible = true;
    TrayIcon->Visible = !Visible;
  }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::OnSelectSources(TObject *Sender)
{
  StartBtn->Enabled = VideoSources->ItemIndex || AudioSources->ItemIndex;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::OnLoad(TObject *Sender)
{
  if(OpenPictureDialog->Execute())
  {
    logo.LoadLogo(WideString(OpenPictureDialog->FileName).c_bstr());
    DrawImage();
  }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::OnClear(TObject *Sender)
{
  OpenPictureDialog->FileName = "";
  Image->Picture = Image2->Picture;
  logo.Abort();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DrawImage()
{
  BITMAPFILEHEADER fh;

  memset (&fh, 0, sizeof(BITMAPFILEHEADER));
  RGBQUAD *src = logo.GetData();

  if(src)
  {
    LPBITMAPINFOHEADER lpbi = logo.GetFormat();
    // prepare header
    fh.bfType = 0x4D42;
    fh.bfOffBits = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER);
    fh.bfSize = fh.bfOffBits + lpbi->biSizeImage;
    // prepare data
    RGBQUAD *dst = new RGBQUAD[lpbi->biWidth * lpbi->biHeight];
    FlipImage(src, dst, lpbi->biWidth, lpbi->biHeight);
    // prepare image
    TMemoryStream *ms = new TMemoryStream();
    ms->Write(&fh, sizeof(BITMAPFILEHEADER));
    ms->Write(lpbi, sizeof(BITMAPINFOHEADER));
    ms->Write(dst, lpbi->biSizeImage);
    ms->Position = 0;
    delete dst;
    // load
    Image->Picture->Bitmap->LoadFromStream(ms);
    delete ms;
  }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FlipImage(RGBQUAD *src,
    RGBQUAD *dst, int width, int height)
{
  for(int i = height - 1, j = 0;i >= 0;i--, j++)
  {
    RGBQUAD *beg = &src[i * width];
    RGBQUAD *end = &dst[j * width];
    memmove(end, beg, width * sizeof(RGBQUAD));
  }
}
//---------------------------------------------------------------------------

int __fastcall TMainForm::GetPosition()
{
  if(TopLeft->Checked) return 0;
  if(TopRight->Checked) return 1;
  if(BottomLeft->Checked) return 2;
  return BottomRight->Checked ? 3 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::SetPosition(int p)
{
  switch(p)
  {
    case 0:
    default:
      TopLeft->Checked = true;
      break;
    case 1:
      TopRight->Checked = true;
      break;
    case 2:
      BottomLeft->Checked = true;
      break;
    case 3:
      BottomRight->Checked = true;
      break;
  }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::OnDrawEnable(TObject *Sender)
{
  cntx.draw = DrawLogo->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::OnOpaque(TObject *Sender)
{
  logo.SetOpaque(Opaque->Checked);
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::OnPosition(TObject *Sender)
{
  logo.SetPosition(GetPosition());
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::OnSettings(TObject *Sender)
{
  PageControl1->Visible = SpeedButton->Down;
  if(PageControl1->Visible)
  {
    Height += PageControl1->Height;
  }
  else
  {
    Height -= PageControl1->Height;
  }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::WatchDogTimer(TObject *Sender)
{
  WatchDog->Enabled = false;
  if(rtsp && rtsp->isNotConnected())
  {
    Stop("Disconnected");
  }
  if(!rtsp && Timer->Enabled)
  {
    if(Start("") != HTTP_OK)
    {
      Stop("Disconnected");
    }
  }
  WatchDog->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ChangeRateClick(TObject *Sender)
{
  FrameRate->Enabled = ChangeRate->Checked;
  Edit4->Enabled = ChangeRate->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::GmaxLogoClick(TObject *Sender)
{
  AnsiString cmd = "rundll32 url.dll,FileProtocolHandler " + GmaxLogo->Hint;
  ::WinExec(cmd.c_str(), SW_SHOW);
}
//---------------------------------------------------------------------------

