/*
* RTP/RTCP H264/MPEG-4 AAC Encoder
* Copyright (c) 2008 Marius Gligor
*
* Author: Marius Gligor <marius.gligor@gmail.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
*/

#ifndef MainH
#define MainH

#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include <Dialogs.hpp>
#include <ComCtrls.hpp>
#include <ToolWin.hpp>
#include <Menus.hpp>
#include <ExtCtrls.hpp>
#include <jpeg.hpp>
#include <Graphics.hpp>
#include <XPMan.hpp>
#include <ImgList.hpp>
#include <Buttons.hpp>
#include "Registry.hpp"
#include "Device.h"
#include "Context.h"
#include "AudioIn.h"
#include "VideoIn.h"
#include "Capture.h"
#include "Sess.h"
#include <AppEvnts.hpp>

class TMainForm : public TForm
{
__published: // IDE-managed Components
  TXPManifest *XPManifest1;
  TSaveDialog *SaveDialog;
  TTimer *Timer;
  TPopupMenu *PopupMenu1;
  TMenuItem *N160x1201;
  TMenuItem *N176x1441;
  TMenuItem *N320x2401;
  TMenuItem *N352x2881;
  TMenuItem *N640x4801;
  TMenuItem *N704x5761;
  TMenuItem *N360x2401;
  TMenuItem *N360x2881;
  TMenuItem *N720x5761;
  TMenuItem *N720x5762;
  TMenuItem *N512x2881;
  TApplicationEvents *ApplicationEvents;
  TMainMenu *MainMenu;
  TMenuItem *Protect1;
  TStatusBar *StatusBar;
  TPageControl *PageControl1;
  TTabSheet *TabSheet2;
  TTabSheet *TabSheet3;
  TTabSheet *TabSheet4;
  TTabSheet *TabSheet5;
  TPanel *Panel1;
  TPanel *Renderer;
  TCheckBox *Preview;
  TOpenDialog *OpenPictureDialog;
  TGroupBox *Position;
  TRadioButton *TopLeft;
  TRadioButton *TopRight;
  TRadioButton *BottomLeft;
  TRadioButton *BottomRight;
  TGroupBox *GroupBox5;
  TRadioButton *Opaque;
  TRadioButton *Transparent;
  TImage *Image2;
  TImage *Image;
  TCheckBox *DrawLogo;
  TButton *LogoButton;
  TButton *ClearButton;
  TLabel *Label22;
  TComboBox *AudioSources;
  TLabel *Label4;
  TLabel *Label8;
  TLabel *Label9;
  TLabel *Label10;
  TComboBox *ObjectType;
  TComboBox *SampleRate;
  TLabel *Label12;
  TComboBox *Channels;
  TEdit *Edit8;
  TUpDown *AudioDataRate;
  TLabel *Label11;
  TLabel *Label3;
  TComboBox *Mode;
  TLabel *Label1;
  TLabel *Label6;
  TEdit *Host;
  TEdit *RtpHost;
  TLabel *Label7;
  TLabel *Label2;
  TEdit *Edit2;
  TEdit *Edit3;
  TUpDown *Port;
  TUpDown *RtpPort;
  TLabel *Label5;
  TLabel *Label24;
  TEdit *Path;
  TEdit *Edit9;
  TUpDown *TTL;
  TLabel *Label29;
  TLabel *Label30;
  TEdit *usr;
  TEdit *psw;
  TLabel *Label21;
  TLabel *Label18;
  TLabel *Label13;
  TLabel *Label14;
  TLabel *Label15;
  TLabel *Label16;
  TComboBox *VideoSources;
  TComboBox *Profile;
  TEdit *Edit5;
  TUpDown *W;
  TLabel *Label23;
  TEdit *Edit6;
  TUpDown *H;
  TLabel *Label20;
  TLabel *Label17;
  TUpDown *FrameRate;
  TEdit *Edit4;
  TEdit *Edit7;
  TUpDown *VideoDataRate;
  TEdit *Edit1;
  TUpDown *KeyFrame;
  TSpeedButton *SpeedButton;
  TToolBar *ToolBar1;
  TButton *StartBtn;
  TButton *VideoBtn;
  TButton *AudioBtn;
  TButton *SdpBtn;
  TTrayIcon *TrayIcon;
  TTimer *WatchDog;
  TTabSheet *TabSheet1;
  TCheckBox *Autorun;
  TCheckBox *WatchdogEnabled;
  TLabel *Label25;
  TEdit *Edit10;
  TUpDown *WatchdogTime;
  TMemo *Memo;
  TLabel *Label27;
  TCheckBox *ChangeRate;
  TMenuItem *N11;
  TMenuItem *N1920x10801;
  TComboBox *Preset;
  TLabel *Label26;
  TComboBox *Tune;
  TLabel *Label28;
  TComboBox *Tune1;
  TComboBox *Tune2;
  TLabel *Label31;
  TImage *GmaxLogo;
  void __fastcall OnClose(TObject *Sender, TCloseAction &Action);
  void __fastcall OnStart(TObject *Sender);
  void __fastcall OnVideo(TObject *Sender);
  void __fastcall OnAudio(TObject *Sender);
  void __fastcall OnSdp(TObject *Sender);
  void __fastcall OnTimer(TObject *Sender);
  void __fastcall OnDestroy(TObject *Sender);
  void __fastcall OnResolution(TObject *Sender);
  void __fastcall OnLock(TObject *Sender);
  void __fastcall OnUnlock(TObject *Sender);
  void __fastcall OnSelectSources(TObject *Sender);
  void __fastcall OnLoad(TObject *Sender);
  void __fastcall OnClear(TObject *Sender);
  void __fastcall OnDrawEnable(TObject *Sender);
  void __fastcall OnOpaque(TObject *Sender);
  void __fastcall OnPosition(TObject *Sender);
  void __fastcall OnSettings(TObject *Sender);
  void __fastcall WatchDogTimer(TObject *Sender);
  void __fastcall FormCreate(TObject *Sender);
  void __fastcall ChangeRateClick(TObject *Sender);
  void __fastcall GmaxLogoClick(TObject *Sender);

private:
  int Page;
  void __fastcall MakeSDP();
  int __fastcall InitializeContext();
  void __fastcall DrawImage();
  void __fastcall FlipImage(RGBQUAD *src, RGBQUAD *dst, int width, int height);
  int __fastcall Start(AnsiString txt);
  void __fastcall Stop(AnsiString txt);
  void Load();
  void Save();
  int IsMulticast(char *ip);
  LPWAVEFORMATEX GetAudioFormat();
  void GetRendererSize();
  int __fastcall GetPosition();
  void __fastcall SetPosition(int p);

  CLogo logo;
  CCapture *graf;
  CSess *rtsp;
  WAVEFORMATEX wfx;
  AVRTP_CONTEXT cntx;
  char sdp[4096];
  int z;

public: // User declarations
  __fastcall TMainForm(TComponent* Owner);
  void __fastcall LoadParams();
  void __fastcall SaveParams();
  void __fastcall StartUp();
  void __fastcall CleanUp();
  void ErrorMessage(DWORD dwError);
};

extern PACKAGE TMainForm *MainForm;
#endif
