/*
* RTP/RTCP H264/MPEG-4 AAC Encoder
* Copyright (c) 2008 Marius Gligor
*
* Author: Marius Gligor <marius.gligor@gmail.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
*/
//---------------------------------------------------------------------------

#include "AACEncoder.h"
//---------------------------------------------------------------------------
// ctor
CAACEncoder::CAACEncoder() : CAac()
{
  // output
  faac_handle = NULL;
}
//---------------------------------------------------------------------------
// dtor
CAACEncoder::~CAACEncoder()
{
}
//---------------------------------------------------------------------------
int32_t CAACEncoder::Open(AVRTP_CONTEXT *cntx)
{
  this->cntx = cntx;
  // RTP
  rtp.Create(cntx->a_host, cntx->a_port, AAC_PAYLOAD_TYPE);
  rtp.SetBufferSize();
  rtp.SetTTL(cntx->ttl);
  // AAC
  faac_handle = _faacEncOpen(cntx->wfx->nSamplesPerSec,
    cntx->wfx->nChannels,
    &samples_input,
    &max_bytes_output);
  if(!faac_handle)
  {
    return -1;
  }
  rtp.SetClock(cntx->wfx->nSamplesPerSec);
  /* get current version */
  faac_cfg = _faacEncGetCurrentConfiguration(faac_handle);
  if(faac_cfg == NULL)
  {
    return -1;
  }
  /* check faac version */
  if(faac_cfg->version != FAAC_CFG_VERSION)
  {
    return -1;
  }
  /* put the options in the configuration struct */
  faac_cfg->aacObjectType = cntx->a_obj; //MAIN, LOW, SSR, LTP
  if (faac_cfg->aacObjectType == SSR) /* Set to LTP */
  {
    faac_cfg->aacObjectType = LTP;
  }
  faac_cfg->mpegVersion = MPEG4;
  faac_cfg->useTns = 0;
  faac_cfg->allowMidside = 1; //1
  faac_cfg->bitRate = cntx->a_bit_rate / cntx->wfx->nChannels;
  faac_cfg->bandWidth = cntx->a_band_width;
  faac_cfg->outputFormat = 0; //0 = ROW, 1 = ADTS
  faac_cfg->inputFormat = FAAC_INPUT_16BIT;
  if(!_faacEncSetConfiguration(faac_handle, faac_cfg))
  {
    return -1;
  }
  /* return frame size */
  return samples_input;
}
//---------------------------------------------------------------------------
int32_t CAACEncoder::Close()
{
  if(faac_handle)
  {
    _faacEncClose(faac_handle);
    faac_handle = NULL;
  }

  rtp.Destroy();

  return 0;
}
//---------------------------------------------------------------------------
#define MAXBYTE 0xff
int32_t CAACEncoder::Encode(int16_t *inp, uint32_t inp_size)
{
  if(!faac_handle)
  {
    return -1;
  }

  // build packet
  index = 0;
  while(inp_size > 0)
  {
    // encode
    int32_t out_size = _faacEncEncode(faac_handle,
      inp, samples_input, tmp, max_bytes_output);
    // prepare for next
    inp      += samples_input;
    inp_size -= samples_input;
    // put on the packet
    if(out_size > 0)
    {
      if(out_size < MAXBYTE)
      {
        out[index++] = out_size;
      }
      else
      {
        out[index++] = MAXBYTE;
        out[index++] = out_size - MAXBYTE;
      }
      memcpy(&out[index], tmp, out_size);
      index += out_size;
    }
  }
  // send packet
  rtp.SendPacket(out, index, 1);
  cntx->audio_bw = bwt.get(index);
  return 0;
}
//---------------------------------------------------------------------------

