/*
* RTP/RTCP H264/MPEG-4 AAC Encoder
* Copyright (c) 2008 Marius Gligor
*
* Author: Marius Gligor <marius.gligor@gmail.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
*/

#ifndef AVCEncoderH
#define AVCEncoderH

#include "context.h"
#include "Bandwidth.h"
#include "Network.h"
#include "avc.h"

#define I420_SIZE(w, h)((w) * (h) * 3 / 2)
#define NORMAL 0
#define FLIP   1

#define MTU 1500
#define MAX_PAYLOAD (3 * MTU)
 /*
   +---------------+
   |0|1|2|3|4|5|6|7|
   +-+-+-+-+-+-+-+-+
   |F|NRI|  Type   |
   +---------------+
*/
#define STAP_A 24
struct NALU_HDR
{
  uint8_t type:5;
  uint8_t nri:2;
  uint8_t f:1;
};
/*
  +---------------+
  |0|1|2|3|4|5|6|7|
  +-+-+-+-+-+-+-+-+
  |S|E|R|  Type   |
  +---------------+
*/
#define FU_A 28
struct FU_HDR
{
  uint8_t type:5;
  uint8_t r:1;
  uint8_t e:1;
  uint8_t s:1;
};


#define Y(r, g, b)((299 * r + 587 * g + 114 * b) / 1000)
#define U(b, y)((b - y) * 565 / 1000 + 128)
#define V(r, y)((r - y) * 713 / 1000 + 128)

#define CLOCK90KHZ 90000
#define AVC_PAYLOAD_TYPE 98

class CAVCEncoder : public CAvc
{
  x264_t *enc;
  x264_param_t params;
  x264_picture_t pic_inp;
  x264_picture_t pic_out;
  int32_t i420_size;
  // convert image
  void RGB2I420P(RGBTRIPLE *rgb, uint8_t flip,
    int32_t width, int32_t height);
  // single and aggregate
  int32_t fragment(x264_nal_t *nal);
  int32_t single(x264_nal_t *nal, int m);
  int32_t aggregate(x264_nal_t *nals,  int32_t nnal);
  int32_t stap_size(x264_nal_t *nals,  int32_t nnal);
  char *tune();
  // RTP
  RTPNetwork rtp;
  Bandwidth bwt;
  AVRTP_CONTEXT *cntx;
public:
  CAVCEncoder();
  virtual ~CAVCEncoder();
  int32_t Open(AVRTP_CONTEXT *cntx);    // initialize
  int32_t EncodeHeaders();
  int32_t Encode(uint8_t *rgb, int32_t rgb_size);
  int32_t Close();
};

#endif
