/*
* RTP/RTCP H264/MPEG-4 AAC Encoder
* Copyright (c) 2008 Marius Gligor
*
* Author: Marius Gligor <marius.gligor@gmail.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
*/

#ifndef CAPTUREH
#define CAPTUREH

#include "BaseCapture.h"

//---------------------------------------------------------------------------

class CCapture : public CBaseCapture
{
  AVRTP_CONTEXT *cntx;

  // video filters
  IBaseFilter *pVideoCapture;
  IBaseFilter *pVideoSampleGrabber;
  IBaseFilter *pVideoRenderer;

  BOOL AddVideoSource();
  BOOL AddVideoGrabber();
  BOOL AddVideoRenderer();
  BOOL SetVideoFormat();
  BOOL ConfigVideoRenderer();
  BOOL ConfigAudioRenderer();

  // audio filters
  IBaseFilter *pAudioCapture;
  IBaseFilter *pAudioSampleGrabber;
  IBaseFilter *pAudioRenderer;

  BOOL AddAudioSource();
  BOOL AddAudioGrabber();
  BOOL AddAudioRenderer();

  BOOL SetBufferSize(IBaseFilter *pCapture);
  BOOL SetAudioFormat(IBaseFilter *pCapture);

  BOOL AddVideoFilters();
  BOOL AddAudioFilters();
  BOOL MakeNormal();

  void ShowError();
  BOOL MakeDVVideo();
  BOOL MakeDVAudio();
  BOOL ConfigDV();
  BOOL ChangeFrameRate(BOOL bHalfFrameRate = TRUE);
  BOOL ChangeVideoSize(int nSize = DVRESOLUTION_HALF);
  int MakeVideoSize(int nSize);

  BOOL ConfigAudioGrabber();
  BOOL ConfigVideoGrabber();
  void Timeout();

public:
  // ctor
  CCapture();
  virtual ~CCapture();
  // graph
  BOOL InitGraph(AVRTP_CONTEXT &p_cntx);
  BOOL VideoProperty(HWND hWnd);
  BOOL AudioProperty(HWND hWnd);
};

//---------------------------------------------------------------------------

#endif;