/*
* RTP/RTCP H264/MPEG-4 AAC Encoder
* Copyright (c) 2008 Marius Gligor
*
* Author: Marius Gligor <marius.gligor@gmail.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
*/

#include "Device.h"

void CDevice::FindSources(HWND hWnd, REFCLSID ClassID)
{
  // Create the System Device Enumerator.
  ICreateDevEnum *pSysDevEnum;
  HRESULT hr = ::CoCreateInstance(CLSID_SystemDeviceEnum,
    NULL,
    CLSCTX_INPROC_SERVER,
    IID_ICreateDevEnum,
    (void **)&pSysDevEnum);
  if(hr != S_OK)
  {
    return;
  }
  // Obtain a class enumerator for the video compressor category.
  IEnumMoniker *pEnumCat;
  hr = pSysDevEnum->CreateClassEnumerator(ClassID, &pEnumCat, 0);
  if(hr != S_OK)
  {
    return;
  }
  // Enumerate the monikers.
  VARIANT varName;
  ULONG cFetched;
  IPropertyBag *pPropBag;
  IMoniker *pMoniker;
  while(pEnumCat->Next(1, &pMoniker, &cFetched) == S_OK)
  {
    hr = pMoniker->BindToStorage(0, 0, IID_IPropertyBag, (void **)&pPropBag);
    if(hr == S_OK)
    {
      // To retrieve the friendly name of the filter, do the following:
      VariantInit(&varName);
      hr = pPropBag->Read(_T("FriendlyName"), &varName, 0);
      if(hr == S_OK)
      {
        ::SendMessage(hWnd, CB_ADDSTRING, 0, (LONG)varName.bstrVal);
      }
      VariantClear(&varName);
      hr = pPropBag->Release();
    }
    hr = pMoniker->Release();
  }
  // clean up
  hr = pEnumCat->Release();
  hr = pSysDevEnum->Release();
}

void CDevice::GetAudioSources(HWND hWnd)
{
  return FindSources(hWnd, CLSID_AudioInputDeviceCategory);
}

void CDevice::GetVideoSources(HWND hWnd)
{
  return FindSources(hWnd, CLSID_VideoInputDeviceCategory);
}

