/*
* RTP/RTCP H264/MPEG-4 AAC Encoder
* Copyright (c) 2008 Marius Gligor
*
* Author: Marius Gligor <marius.gligor@gmail.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
*/
//---------------------------------------------------------------------------
#include "Logo.h"

//---------------------------------------------------------------------------

CLogo::CLogo()
{
  // ctor
  dx = dy = 8;
  bmp = NULL;
  logo = NULL;
  bOpaque = FALSE;

  memset(&bi, 0, sizeof(bi));
  bi.biSize = sizeof(BITMAPINFOHEADER);
  bi.biPlanes = 1;
  bi.biBitCount = 32;

  Gdiplus::GdiplusStartup(&token, &gdi, NULL);
  status = Gdiplus::Aborted;
}
//---------------------------------------------------------------------------

CLogo::~CLogo()
{
  // dtor
  Abort();
  Gdiplus::GdiplusShutdown(token);
}
//---------------------------------------------------------------------------

void CLogo::Abort()
{
  status = Gdiplus::Aborted;
  CancelLogo();
}
//---------------------------------------------------------------------------

void CLogo::CancelLogo()
{
  if(logo)
  {
    delete logo;
    logo = NULL;
  }
}
//---------------------------------------------------------------------------

void CLogo::DrawLogo(RGBTRIPLE *frame, int width, int height)
{
  if(status != Gdiplus::Ok)
  {
    return;
  }
  // draw
  if(!CalcPosition(width, height))
  {
    // cannot draw
    return;
  }
  for(int i = 0;i < y;i++)
  {
    RGBQUAD *src = logo + i * x;
    RGBTRIPLE *dst = frame + (height - dy - i) * width + dx;
    for(int j = 0;j < x;j++)
    {
      if(bOpaque || (!bOpaque && !isTransparentColor(src[j])))
      {
        dst[j].rgbtBlue = src[j].rgbBlue;
        dst[j].rgbtGreen = src[j].rgbGreen;
        dst[j].rgbtRed = src[j].rgbRed;
      }
    }
  }
}
//---------------------------------------------------------------------------

void CLogo::LoadLogo(LPTSTR file)
{
  bmp = new Gdiplus::Bitmap(file);
  if(bmp)
  {
    x = bmp->GetWidth();
    y = bmp->GetHeight();
    if((x <= MAX_DX) && (y <= MAX_DY))
    {
      bmp->RotateFlip(Gdiplus::Rotate180FlipXY);
      Gdiplus::Rect rect(0, 0, x, y);
      status = bmp->LockBits(&rect, Gdiplus::ImageLockModeRead,
        PixelFormat32bppRGB, &bitmapData);
      if(status == Gdiplus::Ok && !GetImage())
      {
        status = Gdiplus::Aborted;
      }
      bmp->UnlockBits(&bitmapData);
    }
    delete bmp;
    bmp = NULL;
  }
}
//---------------------------------------------------------------------------

BOOL CLogo::GetImage()
{
  // release logo
  CancelLogo();
  // alloc
  logo = new RGBQUAD[x * y];
  if(!logo)
  {
    return FALSE;
  }

  BYTE *src = (BYTE *)bitmapData.Scan0;
  BYTE *dst = (BYTE *)logo;

  int line_size = x * sizeof(RGBQUAD);
  int stride = bitmapData.Stride;
  if(stride < 0)
  {
    dst += (y * (line_size - 1));
    src += (y * ((-stride) - 1));
    line_size = -line_size;
  }

  int row = x * sizeof(RGBQUAD);
  for(int i = 0;i < y;i++)
  {
    memmove(dst, src, row);
    dst += line_size;
    src += stride;
  }

  t = logo[0];

  bi.biWidth = x;
  bi.biHeight = y;
  bi.biSizeImage = x * y * sizeof(RGBQUAD);

  return TRUE;
}
//---------------------------------------------------------------------------

BOOL CLogo::isTransparentColor(RGBQUAD r)
{
  // compare blue
  if(r.rgbBlue != t.rgbBlue)
  {
    return FALSE;
  }
  // compare green
  if(r.rgbGreen != t.rgbGreen)
  {
    return FALSE;
  }
  // compare red
  return (r.rgbRed == t.rgbRed);
}
//---------------------------------------------------------------------------

BOOL CLogo::CalcPosition(int width, int height)
{
  switch(mode)
  {
    case 0:
      dx = 8;
      dy = 8;
      break;
    case 1:
      dx = width - x - 8;
      dy = 8;
      break;
    case 2:
      dx = 8;
      dy = height - y - 8;
      break;
    case 3:
      dx = width - x - 8;
      dy = height - y - 8;
      break;
  }
  return ((dx >= 0) && (dy >= 0));
}
//---------------------------------------------------------------------------

