/*
* RTP/RTCP H264/MPEG-4 AAC Encoder
* Copyright (c) 2008 Marius Gligor
*
* Author: Marius Gligor <marius.gligor@gmail.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
*/
//---------------------------------------------------------------------------

#include "RTPTime.h"
//---------------------------------------------------------------------------

RTPTime::RTPTime()
{
  // ctor
  randomize();
}
//---------------------------------------------------------------------------

RTPTime::~RTPTime()
{
  // dtor
}
//---------------------------------------------------------------------------

// clock params
void RTPTime::SetClock(uint32_t clock)
{
  clock_freq = clock;
}
//---------------------------------------------------------------------------

int RTPTime::gettimeofday(struct timeval *tv)
{
  if(tv)
  {
    // ok
    gettimeofday();
    tv->tv_sec  = SS(ft);
    tv->tv_usec = UU(ft);
    return 0;
  }
  return -1;
}
//---------------------------------------------------------------------------

uint64_t RTPTime::gettimeofday()
{
  ::GetSystemTimeAsFileTime((LPFILETIME)&ft);
  /*convert into microseconds*/
  ft /= 10;
  /*converting file time to unix epoch*/
  ft -= DELTA_EPOCH_IN_MICROSECS;
  return ft;
}
//---------------------------------------------------------------------------

uint64_t RTPTime::ntp_time()
{
  return gettimeofday() + NTP_OFFSET_US;
}
//---------------------------------------------------------------------------

void RTPTime::ntp_time(uint32_t &sec, uint32_t &usec, uint64_t time)
{
  sec  = SS(time);
  uint64_t tmp = UU(time) << 32;
  usec = tmp / TIME_BASE; // microseconds
}
//---------------------------------------------------------------------------

uint32_t RTPTime::rescale(int64_t time)
{
  return (time * clock_freq) / TIME_BASE;
}
//---------------------------------------------------------------------------

uint32_t RTPTime::rtp_time()
{
  return base_rtp_time + rescale(ntp_time() - base_ntp_time);
}
//---------------------------------------------------------------------------

uint32_t RTPTime::sr(uint64_t time)
{
  // every 2 seconds
  if((time - last_sr_time) < TWO_SECONDS)
  {
    return 0;
  }
  last_sr_time = time;
  return  1;
}
//---------------------------------------------------------------------------

uint32_t RTPTime::random()
{
#ifdef RANDOM
  ::GetSystemTimeAsFileTime((LPFILETIME)&tmp);
  return ::random(tmp);
#else
  return 0;
#endif
}
//---------------------------------------------------------------------------

