/*
* RTP/RTCP H264/MPEG-4 AAC Encoder
* Copyright (c) 2008 Marius Gligor
*
* Author: Marius Gligor <marius.gligor@gmail.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
*/
//---------------------------------------------------------------------------
#include "rtsp.h"

//---------------------------------------------------------------------------
// ctor
CMsg::CMsg()
{
}
//---------------------------------------------------------------------------
// dtor
CMsg::~CMsg()
{
}
//---------------------------------------------------------------------------
void CMsg::SetMessage(char *verb, char *uri, char *proto)
{
  sprintf(msg, "%s %s %s\r\n", verb, uri, proto);
}
//---------------------------------------------------------------------------
// Headers
//---------------------------------------------------------------------------
// ctor
CHdr::CHdr()
{
}
//---------------------------------------------------------------------------
// dtor
CHdr::~CHdr()
{
}
//---------------------------------------------------------------------------
void CHdr::OpenHeaders()
{
  hdr[0] = 0;
}
//---------------------------------------------------------------------------
void CHdr::CloseHeaders()
{
  strcat(hdr, "\r\n");
}
//---------------------------------------------------------------------------
void CHdr::AddHeaders(char *name, char *value)
{
  sprintf(tmp, "%s: %s\r\n", name, value);
  strcat(hdr, tmp);
}
//---------------------------------------------------------------------------
void CHdr::AddHeaders(char *name, int value)
{
  sprintf(tmp, "%s: %u\r\n", name, value);
  strcat(hdr, tmp);
}
//---------------------------------------------------------------------------
// Rtsp client
//---------------------------------------------------------------------------
// ctor
CRtsp::CRtsp() : CMsg(), CHdr()
{
}
//---------------------------------------------------------------------------
// dtor
CRtsp::~CRtsp()
{
}
//---------------------------------------------------------------------------
int CRtsp::Connect(char *host, int port)
{
  int optval, optlen = sizeof(int), mode = 1;
  // init
  timeout.tv_sec = 10;
  timeout.tv_usec = 0;
  FD_ZERO(&set);
  // server
  ss.sin_family = AF_INET;
  ss.sin_port = htons(port);
  ss.sin_addr.s_addr = inet_addr(host);

  cli = ::socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
  int r = ::connect(cli, (sockaddr *)&ss, sizeof(ss));
  if(r == 0)
  {
    FD_SET(cli, &set);
    ::setsockopt(cli, SOL_SOCKET, SO_KEEPALIVE, (char *)&optval, optlen);
    ::ioctlsocket(cli, FIONBIO, (u_long*)&mode);
  }

  return (r == 0);
}
//---------------------------------------------------------------------------
int CRtsp::Disconnect()
{
  ::shutdown(cli, SD_BOTH);
  ::closesocket(cli);
  return 0;
}
//---------------------------------------------------------------------------
int CRtsp::DoRequest()
{
  int size = strlen(msg) + strlen(hdr);
  char *buf = (char *)malloc(size);
  memcpy(buf, msg, strlen(msg));
  memcpy(buf + strlen(msg), hdr, strlen(hdr));
  ::send(cli, buf, size, 0);
  free(buf);
  return 0;
}
//---------------------------------------------------------------------------
int CRtsp::DoRequest(char *body)
{
  int size = strlen(msg) + strlen(hdr) + strlen(body);
  char *buf = (char *)malloc(size);
  memcpy(buf, msg, strlen(msg));
  memcpy(buf + strlen(msg), hdr, strlen(hdr));
  memcpy(buf + strlen(msg) + strlen(hdr), body, strlen(body));
  ::send(cli, buf, size, 0);
  free(buf);
  return 0;
}
//---------------------------------------------------------------------------
int CRtsp::GetLine(char *line)
{
  int pos = 0;
  // ...
  while(::select(0, &set, 0, 0, &timeout) == 1)
  {
    // find
    unsigned long chunk = 0;
    if(::ioctlsocket(cli, FIONREAD, &chunk) != 0)
    {
      break;
    }
    if(chunk == 0)
    {
      break;
    }
    // read
    for(unsigned i = 0;i < chunk;i++)
    {
      if(::recv(cli, &line[pos], 1, 0) != 1)
      {
        return -1;
      }
      if(line[pos++] == '\n')
      {
        return pos;
      }
    }
  }

  return -1;
}
//---------------------------------------------------------------------------
int CRtsp::DoResponse()
{
  out[0] = 0;
  char *hdr = out;
  while(1)
  {
    int line = GetLine(hdr);
    if(line < 0)
    {
      return 0;
    }
    if(memcmp(hdr, "\r\n", 2) == 0)
    {
      hdr[2] = 0;
      break;
    }
    hdr += line;
  }
  return (sscanf(out, "%s %u %s\r\n", proto, &code, error) == 3) ? code : 0;
}
//---------------------------------------------------------------------------
int CRtsp::isNotConnected()
{
  char ack;
  return ::recv(cli, &ack, 1, MSG_PEEK) == 0;
}
//---------------------------------------------------------------------------
// Receive
//---------------------------------------------------------------------------
unsigned CRec::GetIntAtom(char *name)
{
  char *inp = strstr(out, name);
  if(inp == NULL)
  {
    return 0;
  }
  sprintf(fmt, "%s: %s\r\n", name, "%u");
  return (sscanf(inp, fmt, &i_value)) ? i_value : 0;
}
//---------------------------------------------------------------------------
char *CRec::GetStrAtom(char *name)
{
  char *inp = strstr(out, name);
  if(inp == NULL)
  {
    return 0;
  }
  sprintf(fmt, "%s: %s\r\n", name, "%s");
  return (sscanf(inp, fmt, &s_value)) ? s_value : 0;
}
//---------------------------------------------------------------------------
char *CRec::GetField(char *name)
{
  memset(tmp, 0, sizeof(tmp));
  char *beg = strstr(out, name);
  char *end = strstr(beg, "\r\n");
  if(!beg || !end)
  {
    return 0;
  }
  beg = beg + strlen(name) + 2;
  strncpy(tmp, beg, end - beg);
  return tmp;
}
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------

void CAuth::Init(char *auth)
{
  strcpy(tmp, auth);
  AuthSchema = -1;
  if(strncmp(tmp, "Basic ", 6) == 0)
  {
    AuthSchema = 0;
  }
  else if(strncmp(tmp, "Digest ", 7) == 0)
  {
    AuthSchema = 1;
  }
}
//---------------------------------------------------------------------------

char *CAuth::GetAttribute(char *name)
{
  sprintf(att, "%s=\"", name);

  char *beg = strstr(tmp, att);
  if(!beg)
  {
    return 0;
  }
  beg = beg + strlen(name) + 2;
  char *end = strstr(beg, "\"");
  if(!end)
  {
    return 0;
  }

  memset(att, 0, sizeof(att));
  strncpy(att, beg, end - beg);
  return att;
}
//---------------------------------------------------------------------------

char *CAuth::Basic(char *user, char *pass)
{
  memset(b64, 0, sizeof(b64));
  sprintf(cred, "%s:%s", user, pass);
  encode64(b64, cred, strlen(cred));
  return b64;
}
//---------------------------------------------------------------------------

char *CAuth::Digest(char *user, char *pass, char *meth, char *uri)
{
  char realm[1024], nonce[1024];
  char fmt[] = "username=\"%s\", realm=\"%s\", nonce=\"%s\", uri=\"%s\", response=\"%s\"";
  sprintf(realm, "%s", GetAttribute("realm"));
  sprintf(nonce, "%s", GetAttribute("nonce"));
  memset(md5, 0, sizeof(md5));
  ha1(user, pass, realm);
  ha2(meth, uri);
  sprintf(cred, "%s:%s:%s", a1, nonce, a2);
  sprintf(md5, "%s", MD5(cred, strlen(cred)));
  // final
  sprintf(rsp, fmt, user, realm, nonce, uri, md5);
  return rsp;
}
//---------------------------------------------------------------------------

char *CAuth::ha1(char *user, char *pass, char *realm)
{
  memset(a1, 0, sizeof(a1));
  sprintf(cred, "%s:%s:%s", user, realm, pass);
  sprintf(a1, "%s", MD5(cred, strlen(cred)));
  return a1;
}
//---------------------------------------------------------------------------

char *CAuth::ha2(char *meth, char *uri)
{
  memset(a2, 0, sizeof(a2));
  sprintf(cred, "%s:%s", meth, uri);
  sprintf(a2, "%s", MD5(cred, strlen(cred)));
  return a2;
}
//---------------------------------------------------------------------------

