/*
* RTP/RTCP H264/MPEG-4 AAC Encoder
* Copyright (c) 2008 Marius Gligor
*
* Author: Marius Gligor <marius.gligor@gmail.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
*/
//---------------------------------------------------------------------------

#ifndef RTSPH
#define RTSPH

#include <stdio.h>
#include <winsock2.h>
#include "base64.h"
#include "md5.h"
//---------------------------------------------------------------------------

class CMsg
{
protected:
  char msg[4096];
public:
  CMsg();
  virtual ~CMsg();
  void SetMessage(char *verb, char *uri, char *proto);
};
//---------------------------------------------------------------------------

class CHdr
{
protected:
  char hdr[8192];
  char tmp[1024];
public:
  CHdr();
  virtual ~CHdr();
  // request
  void OpenHeaders();
  void AddHeaders(char *verb, char *value);
  void AddHeaders(char *verb, int value);
  void CloseHeaders();
};
//---------------------------------------------------------------------------

class CRec
{
protected:
  // response
  char out[4096];
  // errors
  int code;
  char proto[256];
  char error[1024];
  // headers
  char fmt[1024];
  unsigned i_value;
  char s_value[1024];
  char tmp[512];
public:
  CRec() {};
  virtual ~CRec() {};
  unsigned GetIntAtom(char *name);
  char *GetStrAtom(char *name);
  char *GetField(char *name);
  char *GetMsgError() { return error; };
};
//---------------------------------------------------------------------------

class CRtsp : public CMsg, public CHdr, public CRec
{
  // host
  SOCKET cli;
  sockaddr_in ss;

  fd_set set;
  timeval timeout;

  int GetLine(char *line);
public:
  CRtsp();
  virtual ~CRtsp();
  // client
  int Connect(char *host, int port);
  int Disconnect();
  // request
  int DoRequest();
  int DoRequest(char *body);
  // response
  int DoResponse();
  int isNotConnected();
};
//---------------------------------------------------------------------------

#define AUTH_BASIC 0
#define AUTH_DIGEST 1

class CAuth
{
  char tmp[1024];
  char cred[1024];
  char att[512];
  char b64[1024];

  // digest
  char a1[32+1];
  char a2[32+1];
  char md5[32+1];
  char *ha1(char *user, char *pass, char *realm);
  char *ha2(char *meth, char *uri);
  char rsp[1024];
protected:
  int AuthSchema;
public:
  CAuth() {};
  virtual ~CAuth() {};
  void Init(char *auth);
  char *GetAttribute(char *name);
  char *Basic(char *user, char *pass);
  char *Digest(char *user, char *pass, char *meth, char *uri);
};
//---------------------------------------------------------------------------
#endif
