/*
* RTP/RTCP H264/MPEG-4 AAC Encoder
* Copyright (c) 2008 Marius Gligor
*
* Author: Marius Gligor <marius.gligor@gmail.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
*/

#include "sess.h"

//---------------------------------------------------------------------------
CSess::CSess()
{
}
//---------------------------------------------------------------------------
CSess::~CSess()
{
}
//---------------------------------------------------------------------------
int CSess::Open(AVRTP_CONTEXT *c)
{
  z = 0;
  cntx = c;
  if(cntx->mode)
  {
    return HTTP_OK;
  }
  sprintf(uri, "rtsp://%s:%u/%s", cntx->host, cntx->port, cntx->path);
  if(Connect(cntx->host, cntx->port))
  {
    seq = 1;
    Play();
    z = Announce();
    // authenticate
    if(z == HTTP_AUTH)
    {
      Authorization();
      strcpy(sess_id, GetStrAtom("Session"));
      z = Announce(sess_id);
    }
    // setup track
    if(z == HTTP_OK && SetupTrack1() == HTTP_OK && SetupTrack2() == HTTP_OK)
    {
      Play();
    }
  }
  else
  {
    cntx->ok = ::WSAGetLastError();
  }
  return z;
}
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
int CSess::Close()
{
  if(!cntx->mode)
  {
    if(z == HTTP_OK)
    {
      Stop();
    }
    Disconnect();
  }
  return 0;
}
//---------------------------------------------------------------------------
int CSess::Authorization()
{
  Init(GetField("WWW-Authenticate"));
  strcpy(cntx->realm, GetAttribute("realm"));
  switch(AuthSchema)
  {
    case AUTH_BASIC:
      sprintf(key, "Basic %s",
      Basic(cntx->username, cntx->password));
      break;
    case AUTH_DIGEST:
      sprintf(key, "Digest %s",
      Digest(cntx->username, cntx->password, "ANNOUNCE", uri));
      break;
    default:
      strcpy(key, "None");
      break;
  }
  return 0;
}
//---------------------------------------------------------------------------
int CSess::Announce()
{
  SetMessage("ANNOUNCE", uri, "RTSP/1.0");
  OpenHeaders();
  AddHeaders("CSeq", seq);
  AddHeaders("Content-Type", "application/sdp");
  AddHeaders("Content-Length", strlen(cntx->sdp));
  CloseHeaders();
  DoRequest(cntx->sdp);
  z = DoResponse();
  return z;
}
//---------------------------------------------------------------------------
int CSess::Announce(char *sid)
{
  SetMessage("ANNOUNCE", uri, "RTSP/1.0");
  OpenHeaders();
  AddHeaders("CSeq", seq);
  AddHeaders("Session", sid);
  AddHeaders("Authorization", key);
  AddHeaders("Content-Type", "application/sdp");
  AddHeaders("Content-Length", strlen(cntx->sdp));
  CloseHeaders();
  DoRequest(cntx->sdp);
  z = DoResponse();
  return z;
}
//---------------------------------------------------------------------------
int CSess::SetupTrack1()
{
  char tmp[1024];
  int port1 = 0, port2 = 0;

  strcpy(tmp, uri);
  strcat(tmp, "/trackID=1");

  SetMessage("SETUP", tmp, "RTSP/1.0");
  OpenHeaders();
  AddHeaders("CSeq", ++seq);
  AddHeaders("Session", sess_id);
  if(cntx->cast)
  {
    sprintf(tmp, "RTP/AVP;multicast;destination=%s;ttl=%u;client_port=%u-%u",
      cntx->v_host, cntx->ttl, cntx->v_port, cntx->v_port + 1);
  }
  else
  {
    sprintf(tmp, "RTP/AVP;unicast;client_port=%u-%u",
      cntx->v_port, cntx->v_port + 1);
  }
  AddHeaders("Transport", tmp);
  CloseHeaders();
  DoRequest();
  z = DoResponse();

  if(z == HTTP_OK)
  {
    strcpy(tmp, GetStrAtom("Transport"));
    char *fmt = strstr(tmp, "server_port=");
    if(fmt)
    {
      if(sscanf(fmt, "server_port=%u-%u", &port1, &port2) == 2)
      {
        cntx->v_port = port1;
      }
    }
  }
  return z;
}
//---------------------------------------------------------------------------
int CSess::SetupTrack2()
{
  char tmp[1024];
  int port1 = 0, port2 = 0;

  strcpy(tmp, uri);
  strcat(tmp, "/trackID=2");

  SetMessage("SETUP", tmp, "RTSP/1.0");
  OpenHeaders();
  AddHeaders("CSeq", ++seq);
  AddHeaders("Session", sess_id);
  if(cntx->cast)
  {
    sprintf(tmp, "RTP/AVP;multicast;destination=%s;ttl=%u;client_port=%u-%u",
      cntx->a_host, cntx->ttl, cntx->a_port, cntx->a_port + 1);
  }
  else
  {
    sprintf(tmp, "RTP/AVP;unicast;client_port=%u-%u",
      cntx->a_port, cntx->a_port + 1);
  }
  AddHeaders("Transport", tmp);
  CloseHeaders();
  DoRequest();
  z = DoResponse();
  if(z == HTTP_OK)
  {
    strcpy(tmp, GetStrAtom("Transport"));
    char *fmt = strstr(tmp, "server_port=");
    if(fmt)
    {
      if(sscanf(fmt, "server_port=%u-%u", &port1, &port2) == 2)
      {
        cntx->a_port = port1;
      }
    }
  }
  return z;
}
//---------------------------------------------------------------------------
int CSess::Play()
{
  SetMessage("PLAY", uri, "RTSP/1.0");
  OpenHeaders();
  AddHeaders("CSeq", ++seq);
  AddHeaders("Session", sess_id);
  CloseHeaders();
  DoRequest();
  z = DoResponse();
  return z;
}
//---------------------------------------------------------------------------
int CSess::Stop()
{
  SetMessage("TEARDOWN", uri, "RTSP/1.0");
  OpenHeaders();
  AddHeaders("CSeq", ++seq);
  AddHeaders("Session", sess_id);
  CloseHeaders();
  DoRequest();
  z = DoResponse();
  return z;
}
//---------------------------------------------------------------------------



